package idi.diplom.linda.db.implementation;

import idi.diplom.linda.db.interfaces.AlarmDAO;
import idi.diplom.linda.model.Alarm;
import idi.diplom.linda.model.Patient;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.hibernate.criterion.Restrictions;


public class AlarmDAOHibernate extends GenericHibernateDAO<Alarm, Long>
		implements AlarmDAO {

	@Override
	@SuppressWarnings("unchecked")
	public List<Alarm> getAlarmsBetweenDate(Patient p, Date fromTime, Date toTime) {
		List<Alarm> retList = getSession().createCriteria(Alarm.class)
		.add(Restrictions.eq("patient", p))
		.add(Restrictions.between("timestamp", fromTime, toTime))
		.list();
		return retList;
	}

	@Override
	@SuppressWarnings("unchecked")
	public Alarm getAlarmFromTrigger(Patient p, int s) {
		Alarm a = (Alarm)getSession().createCriteria(Alarm.class)
		.add(Restrictions.eq("patient", p))
		.add(Restrictions.eq("triggeredThreshold", s))
		.add(Restrictions.isNull("dismissTime"))
		.uniqueResult();
		
		return a;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Alarm> getAllActiveAlarms(Patient p) {
		List<Alarm> retList = getSession().createCriteria(Alarm.class)
		.add(Restrictions.eq("patient", p))
		.add(Restrictions.isNull("dismissTime"))
		.list();
		return retList;
	}

	@Override
	@SuppressWarnings("unchecked")
	public List<Alarm> getAllDismissedAlarmsWithinInterval(Patient p) {
		int interval = p.getAlarmInterval()*60*1000;
		Date now = Calendar.getInstance().getTime();
		Calendar then = Calendar.getInstance();
		then.setTimeInMillis(then.getTimeInMillis()-interval);
		List<Alarm> retList = getSession().createCriteria(Alarm.class)
		.add(Restrictions.eq("patient", p))
		.add(Restrictions.isNotNull("dismissTime"))
		.add(Restrictions.between("dismissTime", then.getTime(), now))
		.list();
		return retList;
	}

	@Override
	@SuppressWarnings("unchecked")
	public boolean dismissedWithinInterval(long milliseconds, Patient p) {
		Date now = Calendar.getInstance().getTime();
		Calendar then = Calendar.getInstance();
		then.setTimeInMillis(then.getTimeInMillis()-milliseconds);
		List<Alarm> retList = getSession().createCriteria(Alarm.class)
		.add(Restrictions.eq("patient", p))
		.add(Restrictions.isNotNull("dismissTime"))
		.add(Restrictions.between("dismissTime", then.getTime(), now))
		.list();
		if(retList!=null && retList.size()>0){
			return true;
		}
		return false;
	}

}
