package idi.diplom.linda.db.interfaces;

import idi.diplom.linda.model.Alarm;
import idi.diplom.linda.model.Patient;

import java.util.Date;
import java.util.List;


public interface AlarmDAO extends GenericDAO<Alarm, Long>{
	/**
	 * Gets all alarms on the specified patient between the to specified dates
	 * @param Patient p
	 * @param Date fromTime
	 * @param Date toTime
	 * @return List<Alarm> alarmList
	 */
	public List<Alarm> getAlarmsBetweenDate(Patient p,Date fromTime, Date toTime);
	
	/**
	 * @see AlarmLogic
	 * @param Patient p
	 * @param int s
	 * @return Alarm alarm
	 */
	public Alarm getAlarmFromTrigger(Patient p, int s);
	
	/**
	 * @see AlarmLogic
	 * @param Patient p
	 * @return List<Alarm> alarmList
	 */
	public List<Alarm> getAllActiveAlarms(Patient p);
	
	/**
	 * NOT IN USE IN THIS VERSION
	 * @param p
	 * @return
	 */
	public List<Alarm> getAllDismissedAlarmsWithinInterval(Patient p);
	
	/**
	 * Checks if the patient has any alarms that was dismissed within the specified number of milliseconds.
	 * If so, true is returned.
	 * @param long milliseconds
	 * @param patient to check
	 * @return boolean dismissedWithin
	 */
	public boolean dismissedWithinInterval(long milliseconds, Patient p);
}
