package idi.diplom.linda.lifeshirt;

import idi.diplom.linda.db.implementation.HibernateUtil;
import idi.diplom.linda.db.interfaces.DAOFactory;
import idi.diplom.linda.db.interfaces.MedPacketDAO;
import idi.diplom.linda.exceptions.DatabaseException;
import idi.diplom.linda.exceptions.GeneralException;
import idi.diplom.linda.exceptions.LifeShirtException;
import idi.diplom.linda.logic.AlarmLogic;
import idi.diplom.linda.logic.PatientLogic;
import idi.diplom.linda.model.MedPacket;
import idi.diplom.linda.model.Patient;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import org.apache.log4j.Logger;
import org.hibernate.HibernateException;



public class Processer {

	private Logger log = Logger.getLogger(getClass());
	private PatientLogic pl = new PatientLogic();
	private AlarmLogic al = new AlarmLogic();
	
	/**
	 * Takes in a medpacket as a String.
	 * This string is splitted in parts and each part is processed.
	 * The string is converted to a MedPacket object, which is stored in the database.
	 * @param String vars
	 * @throws LifeShirtException
	 */
	public void processMedPacket(String vars) throws LifeShirtException{
		MedPacket mp = new MedPacket();
		Patient patient = null;
		if (vars == null)
			return;
		String[] splits = vars.split("&");
		for(int i=0; i<splits.length; i++){
			//System.out.println(splits[i]);
//			if(splits[i].startsWith("varsloaded")){
//				String[] newSplit = splits[i].split("=");
//				int varsloaded = Integer.parseInt(newSplit[1]);
//				mp.setVarsloaded(varsloaded);
//			}
			//ID=1&TIMESTAMP=1206439398&SEQUENCENUMBER=0&HEATIDX=45&BR=0&HR=70&SPO2=98&MOTION=49&TEMP=986&POSTURE=150
			
			if(splits[i].startsWith("ID")){
				String[] newSplit = splits[i].split("=");
				int patientId = Integer.parseInt(newSplit[1]);
				try{
					patient = pl.getPatient(patientId);
				}
				catch(Exception e){
					log.error("No patient on that lifeshirtId: " + patientId);
					throw new LifeShirtException("No patient on received lifeshirtId");
				}
				mp.setPatient(patient);
			}

			else if(splits[i].startsWith("TIMESTAMP")){
				String[] newSplit = splits[i].split("=");
				TimeZone tz = TimeZone.getDefault();
				Calendar instance = Calendar.getInstance();
				instance.setTimeZone(tz);
				Date timestamp = instance.getTime();
				//Date timestamp = Calendar.getInstance().getTime();
				mp.setTimestamp(timestamp);
			}
			else if(splits[i].startsWith("SEQUENCENUMBER")){
				String[] newSplit = splits[i].split("=");
				int seqNo = Integer.parseInt(newSplit[1]);
				mp.setSequenceNumber(seqNo);
			}
			else if(splits[i].startsWith("HEATIDX")){
				String[] newSplit = splits[i].split("=");
				int heatIdx = Integer.parseInt(newSplit[1]);
				mp.setHeatIdx(heatIdx);
			}
			else if(splits[i].startsWith("BR")){
				String[] newSplit = splits[i].split("=");
				int br = Integer.parseInt(newSplit[1]);
				mp.setBr(br);
			}
			else if(splits[i].startsWith("HR")){
				String[] newSplit = splits[i].split("=");
				int hr = Integer.parseInt(newSplit[1]);
				mp.setHr(hr);
			}
			else if(splits[i].startsWith("SPO2")){
				String[] newSplit = splits[i].split("=");
				int spo2 = Integer.parseInt(newSplit[1]);
				mp.setSpo2(spo2);
			}
			else if(splits[i].startsWith("MOTION")){
				String[] newSplit = splits[i].split("=");
				int motion = Integer.parseInt(newSplit[1]);
				mp.setMotion(motion);
			}
			else if(splits[i].startsWith("TEMP")){
				String[] newSplit = splits[i].split("=");
				int temp = Integer.parseInt(newSplit[1]);
				temp /= 10; //from lifeshirt
				double celcius = (temp-32)*(5.0/9.0);
				mp.setTemp(celcius);
			}
			else if(splits[i].startsWith("POSTURE")){
				String[] newSplit = splits[i].split("=");
				int posture = Integer.parseInt(newSplit[1]);
				mp.setPosture(posture);	
			}
			//add more here if new sensors are added
			else{
				System.out.println("error");
				//unknown
				log.error("Text from medpacket is unknown");
				throw new LifeShirtException("Unknown text from medpacket");
			}
		}
		log.debug("Processed Medpacket:\n" + mp);
		
		if(patient!=null){
			try{
				storeMedPacket(mp);
				al.checkForAlarms(patient, mp);
				//al.addPacketToDismissedAlarms(mp, patient);
			}
			catch(DatabaseException db){
				log.error("DatabaseException " + db.getStackTrace());
				throw new LifeShirtException("Database error when processing medpacket to database", db);
			}
			catch(GeneralException g){
				log.error("GeneralException " + g.getStackTrace());
				throw new LifeShirtException("Unknown error when processing medpacket", g);
			}
		}
	}
	
	/**
	 * Stores a medpacket in the database
	 * @param MedPacket mp
	 * @throws DatabaseException
	 */
	public void storeMedPacket(MedPacket mp) throws DatabaseException{
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			MedPacketDAO mpdao = DAOFactory.instance(DAOFactory.HIBERNATE).getMedPacketDAO();		
			mpdao.makePersistent(mp);		
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();		
			log.debug("Stored medpacket in db");
		}
		catch(HibernateException he){
			throw new DatabaseException("Error when storing medpacket", he);
		}
	}

}
