package idi.diplom.linda.logic;

import idi.diplom.linda.db.implementation.HibernateUtil;
import idi.diplom.linda.db.interfaces.AlarmDAO;
import idi.diplom.linda.db.interfaces.DAOFactory;
import idi.diplom.linda.db.interfaces.PatientDAO;
import idi.diplom.linda.exceptions.DatabaseException;
import idi.diplom.linda.exceptions.GeneralException;
import idi.diplom.linda.model.Alarm;
import idi.diplom.linda.model.MedPacket;
import idi.diplom.linda.model.Patient;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.hibernate.HibernateException;

public class AlarmLogic {
	private Date d;
	private MedPacketLogic mpl = new MedPacketLogic();
	private int spo2Count = 0;
	private PatientLogic pl = new PatientLogic();
	//public static Date dismissInterval;

	public AlarmLogic(){
	
	}
	
	//NOT IN USE!
	public Alarm[] getAllAlarms() throws DatabaseException, GeneralException{
		List<Alarm> aList = null;
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			AlarmDAO adao = DAOFactory.instance(DAOFactory.HIBERNATE).getAlarmDAO();		
			aList = adao.findAll();
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();		
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when calling method getAllAlarms()", he);
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
		Alarm[] aArray = new Alarm[aList.size()];
		aArray = aList.toArray(aArray);
		return aArray;
	}
	
	/**
	 * Gets all active alarms (alarms that have not been dismissed) on patient p
	 * @param patient to get alarms from
	 * @return Alarm[] alarmArray
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public Alarm[] getAllActiveAlarms(Patient p) throws DatabaseException, GeneralException{
		List<Alarm> aList = null;
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			AlarmDAO adao = DAOFactory.instance(DAOFactory.HIBERNATE).getAlarmDAO();	
			PatientDAO pdao = DAOFactory.instance(DAOFactory.HIBERNATE).getPatientDAO();
			Patient pap =	pdao.findById(p.getId(), false);
			aList = adao.getAllActiveAlarms(pap);
			//System.out.println(aList.size());
			
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();		
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when calling method getAllActiveAlarms()", he);
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
		
		for(int i=0; i<aList.size(); i++){
			Alarm a = aList.get(i);
			Calendar c = Calendar.getInstance();
			//setter at hvis alarmen er eldre enn 10 minutter
			c.setTimeInMillis(c.getTimeInMillis() - 600000);
			Date old = c.getTime();
			TimeZone tz = TimeZone.getDefault();
			Calendar instance = Calendar.getInstance();
			instance.setTimeZone(tz);
			Date dismissTime = instance.getTime();
			if(a.getTimestamp().before(old)){
				a.setDismissNote("Old alarm, automatically dismissed by the system");
				a.setDismissTime(dismissTime);
				this.dismissAlarm(a);
				aList.remove(i);
			}
		}
		
		Alarm[] aArray = new Alarm[aList.size()];

		aArray = aList.toArray(aArray);
		return aArray;
	}
	
	/**
	 * Gets an alarm on specified patient with specified vital sign
	 * @param patient to get alarm from
	 * @param sensornumber
	 * @return Alarm alarm
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public Alarm getAlarmFromTrigger(Patient p, int s) throws DatabaseException, GeneralException{
		Alarm a = null;
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			AlarmDAO adao = DAOFactory.instance(DAOFactory.HIBERNATE).getAlarmDAO();		
			a = adao.getAlarmFromTrigger(p, s);
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();	
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when calling method getAlarmFromTrigger()", he);			
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
		return a;
	}
	
	/**
	 * Checks if medpacket contains abnormal vital sign values and creates alarms
	 * @param patient to check for alarms on
	 * @param medpacket to check for abnormal values
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public void checkForAlarms(Patient p, MedPacket mp) throws DatabaseException, GeneralException{
		Date now = Calendar.getInstance().getTime();
		boolean dismissedWithin = false;
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			AlarmDAO adao = DAOFactory.instance(DAOFactory.HIBERNATE).getAlarmDAO();		
			dismissedWithin = adao.dismissedWithinInterval(60000, p);
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();	
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when checking for dismissed alarms", he);			
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
		if(!dismissedWithin){
			//System.out.println("kommer hit..");
			TimeZone tz = TimeZone.getDefault();
			Calendar instance = Calendar.getInstance();
			instance.setTimeZone(tz);
			d = instance.getTime();
			
			this.checkHighBR(p, mp);
			this.checkLowBR(p, mp);
			this.checkHighHR(p, mp);
			this.checkLowHR(p, mp);
			//FIXME hy spo2 undvendig
			//this.checkHighSPO2(p, mp);
			this.checkLowSPO2(p, mp);
		}
		else{
			//System.out.println("sjekker ikke for alarm.." + now.toString());
		}
	}
	
	/**
	 * Saves an alarm
	 * @param alarm to be saved
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public void saveAlarm(Alarm a) throws DatabaseException, GeneralException{

		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			AlarmDAO adao = DAOFactory.instance(DAOFactory.HIBERNATE).getAlarmDAO();		
			Date date = a.getDismissTime();
			
			adao.makePersistent(a);
			
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();	
		}
		catch(HibernateException he){
			System.out.println("error hibernateexception");
			throw new DatabaseException("Error caused by hibernate when calling method saveAlarm()", he);			
		}
		catch(Exception e){
			System.out.println("error other exception");
			throw new GeneralException("Unknown error", e);
		}
	}
	
	/**
	 * NOT IN USE IN THIS VERSION
	 * @param mp
	 * @param p
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public void addPacketToDismissedAlarms(MedPacket mp, Patient p) throws DatabaseException, GeneralException{
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			AlarmDAO adao = DAOFactory.instance(DAOFactory.HIBERNATE).getAlarmDAO();		
			
			List<Alarm> alarmList = adao.getAllDismissedAlarmsWithinInterval(p);
			for(Alarm a : alarmList){
				a.addMedPacket(mp);
			}
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when calling method addPacketToDismissedAlarms()", he);			
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
	}
	
	/**
	 * Dismisses the specified alarm
	 * @param alarm to be dismissed
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public void dismissAlarm(Alarm a) throws DatabaseException, GeneralException{
		Patient p = pl.getPatient(a.getPatient().getLifeshirtId());
		a.setPatient(p);
		//because of date problems when sending from client to server the timestamp is set here
		//creates a timezone object from the timezone where the computer runs
		//sets a date time to be now in the specified timezone
		TimeZone tz = TimeZone.getDefault();
		Calendar instance = Calendar.getInstance();
		instance.setTimeZone(tz);
		Date dismissTime = instance.getTime();
		a.setDismissTime(dismissTime);
		//this.fillPacketsInFront(a);
		this.saveAlarm(a);
	}
	
	/**
	 * NOT IN USE IN THIS VERSION
	 * @param a
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	//method to connect medpackets monitored to an alarm before the alarm went off. 
	//based on alarm interval specified by the patient.
	private void fillPacketsInFront(Alarm a) throws DatabaseException, GeneralException{
		List<MedPacket> mpList = mpl.getMedPacketsFromIntervalInFront(a);
		if(mpList!=null && mpList.size()>0){
			MedPacket[] oldArray = a.getMedPackets();
			
			MedPacket[] newList = new MedPacket[mpList.size()+a.getMedPackets().length];
			for(int i=0; i<mpList.size(); i++){
				newList[i] = mpList.get(i);
			}
			for(int i=mpList.size(); i<newList.length; i++){
				newList[i] = oldArray[i-mpList.size()];
			}
			a.setMedPackets(newList);
		}
		this.saveAlarm(a);
	}
	
	private void checkHighBR(Patient p, MedPacket mp) throws DatabaseException, GeneralException{	
		Alarm a = this.getAlarmFromTrigger(p, Thresholds.BR_WARN_HIGH);
		Alarm a2 = this.getAlarmFromTrigger(p, Thresholds.BR_ALARM_HIGH);
		
		//Date d = Calendar.getInstance().getTime();
		if(a!=null && a2!=null){
			//TODO! LOG
			System.out.println("Something is wrong, there should not be two active alarms here");
		}
		else if(a!=null){
			//already active high warn alarm
			if(mp.getBr()>p.getBrThresAlarmHigh()){
			//new alarm must be created and the old dismissed because of the increased value
				a.setDismissNote("BR increased to alarm value");
				a.setDismissTime(d);
				//this.fillPacketsInFront(a);
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.BR_ALARM_HIGH);
				newA.addMedPacket(mp);
				this.saveAlarm(a);
				this.saveAlarm(newA);
			}
			else{
				a.addMedPacket(mp);
				this.saveAlarm(a);
			}
		}
		else if(a2!=null){
			//already active high alarm alarm
			a2.addMedPacket(mp);
			this.saveAlarm(a2);
		}
		else{
		//no active high br alarm
			if(mp.getBr()>p.getBrThresAlarmHigh()){
				//create new high alarm alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.BR_ALARM_HIGH);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else if(mp.getBr()>p.getBrThresWarnHigh()){
				//create new high warn alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.BR_WARN_HIGH);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else{
				//no high br alarm
			}
		}
	
	}
	
	
	private void checkLowBR(Patient p, MedPacket mp) throws DatabaseException, GeneralException{
		Alarm a = this.getAlarmFromTrigger(p, Thresholds.BR_WARN_LOW);
		Alarm a2 = this.getAlarmFromTrigger(p, Thresholds.BR_ALARM_LOW);
		//Date d = Calendar.getInstance().getTime();
		if(a!=null && a2!=null){
			System.out.println("Something is wrong, there should not be two active alarms here");
		}
		else if(a!=null){
			//already active low warn alarm
			if(mp.getBr()<p.getBrThresAlarmLow()){
				//new alarm must be created and the old dismissed because of the decreased value
				a.setDismissNote("BR decreased to alarm value");
				a.setDismissTime(d);
				//this.fillPacketsInFront(a);
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.BR_ALARM_LOW);
				newA.addMedPacket(mp);
				this.saveAlarm(a);
				this.saveAlarm(newA);
			}
			else{
				a.addMedPacket(mp);
				this.saveAlarm(a);
			}
		}
		else if(a2!=null){
			//already active low alarm alarm
			a2.addMedPacket(mp);
			this.saveAlarm(a2);
		}
		else{
			//no active low br alarm
			if(mp.getBr()<p.getBrThresAlarmLow()){
				//create new low alarm alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.BR_ALARM_LOW);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else if(mp.getBr()<p.getBrThresWarnLow()){
				//create new low warn alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.BR_WARN_LOW);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else{
				//no low br alarm
			}
		}
		
		
	}
	
	private void checkHighHR(Patient p, MedPacket mp) throws DatabaseException, GeneralException{
		Alarm a = this.getAlarmFromTrigger(p, Thresholds.HR_WARN_HIGH);
		Alarm a2 = this.getAlarmFromTrigger(p, Thresholds.HR_ALARM_HIGH);
		//Date d = Calendar.getInstance().getTime();
		if(a!=null && a2!=null){
			System.out.println("Something is wrong, there should not be two active alarms here");
		}
		else if(a!=null){
			//already active high warn alarm
			if(mp.getHr()>p.getHrThresAlarmHigh()){
			//new alarm must be created and the old dismissed because of the increased value
				a.setDismissNote("HR increased to alarm value");
				a.setDismissTime(d);
				//this.fillPacketsInFront(a);
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.HR_ALARM_HIGH);
				newA.addMedPacket(mp);
				this.saveAlarm(a);
				this.saveAlarm(newA);
			}
			else{
				a.addMedPacket(mp);
				this.saveAlarm(a);
			}
		}
		else if(a2!=null){
			//already active high alarm alarm
			a2.addMedPacket(mp);
			this.saveAlarm(a2);
		}
		else{
		//no active high hr alarm
			if(mp.getHr()>p.getHrThresAlarmHigh()){
				//create new high alarm alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.HR_ALARM_HIGH);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else if(mp.getHr()>p.getHrThresWarnHigh()){
				//create new high warn alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.HR_WARN_HIGH);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else{
				//no high hr alarm
			}
		}
	}
	
	private void checkLowHR(Patient p, MedPacket mp) throws DatabaseException, GeneralException{
		Alarm a = this.getAlarmFromTrigger(p, Thresholds.HR_WARN_LOW);
		Alarm a2 = this.getAlarmFromTrigger(p, Thresholds.HR_ALARM_LOW);
		//Date d = Calendar.getInstance().getTime();
		if(a!=null && a2!=null){
			System.out.println("Something is wrong, there should not be two active alarms here");
		}
		else if(a!=null){
			//already active low warn alarm
			if(mp.getHr()<p.getHrThresAlarmLow()){
				//new alarm must be created and the old dismissed because of the decreased value
				a.setDismissNote("HR decreased to alarm value");
				a.setDismissTime(d);
				//this.fillPacketsInFront(a);
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.HR_ALARM_LOW);
				newA.addMedPacket(mp);
				this.saveAlarm(a);
				this.saveAlarm(newA);
			}
			else{
				a.addMedPacket(mp);
				this.saveAlarm(a);
			}
		}
		else if(a2!=null){
			//already active low alarm alarm
			a2.addMedPacket(mp);
			this.saveAlarm(a2);
		}
		else{
			//no active low hr alarm
			if(mp.getHr()<p.getHrThresAlarmLow()){
				//create new low alarm alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.HR_ALARM_LOW);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else if(mp.getHr()<p.getHrThresWarnLow()){
				//create new low warn alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.HR_WARN_LOW);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else{
				//no low hr alarm
			}
		}
	}
	
	//NOT IN USE! not meaningful to have high spo2
	private void checkHighSPO2(Patient p, MedPacket mp) throws DatabaseException, GeneralException{
		Alarm a = this.getAlarmFromTrigger(p, Thresholds.SPO2_WARN_HIGH);
		Alarm a2 = this.getAlarmFromTrigger(p, Thresholds.SPO2_ALARM_HIGH);
		//Date d = Calendar.getInstance().getTime();
		if(a!=null && a2!=null){
			System.out.println("Something is wrong, there should not be two active alarms here");
		}
		else if(a!=null){
			//already active high warn alarm
			if(mp.getSpo2()>p.getSpo2ThresAlarmHigh()){
			//new alarm must be created and the old dismissed because of the increased value
				a.setDismissNote("SPO2 increased to alarm value");
				a.setDismissTime(d);
				//this.fillPacketsInFront(a);
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.SPO2_ALARM_HIGH);
				newA.addMedPacket(mp);
				this.saveAlarm(a);
				this.saveAlarm(newA);
			}
			else{
				a.addMedPacket(mp);
				this.saveAlarm(a);
			}
		}
		else if(a2!=null){
			//already active high alarm alarm
			a2.addMedPacket(mp);
			this.saveAlarm(a2);
		}
		else{
		//no active high SPO2 alarm
			if(mp.getSpo2()>p.getSpo2ThresAlarmHigh()){
				//create new high alarm alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.SPO2_ALARM_HIGH);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else if(mp.getSpo2()>p.getSpo2ThresWarnHigh()){
				//create new high warn alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.SPO2_WARN_HIGH);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else{
				//no high SPO2 alarm
			}
		}
	}
	
	private void checkLowSPO2(Patient p, MedPacket mp) throws DatabaseException, GeneralException{
		if (p.getSpo2ThresWarnLow()==0 && p.getSpo2ThresAlarmLow()==0)
			return;
		
		if(mp.getSpo2()==-1 && spo2Count<10){
			spo2Count++;
			return;
		}
		spo2Count=0;
		Alarm a = this.getAlarmFromTrigger(p, Thresholds.SPO2_WARN_LOW);
		Alarm a2 = this.getAlarmFromTrigger(p, Thresholds.SPO2_ALARM_LOW);
		//Date d = Calendar.getInstance().getTime();
		if(a!=null && a2!=null){
			System.out.println("Something is wrong, there should not be two active alarms here");
		}
		else if(a!=null){
			//already active low warn alarm
			if(mp.getSpo2()<p.getSpo2ThresAlarmLow()){
				//new alarm must be created and the old dismissed because of the decreased value
				a.setDismissNote("SPO2 decreased to alarm value");
				a.setDismissTime(d);
				//this.fillPacketsInFront(a);
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.SPO2_ALARM_LOW);
				newA.addMedPacket(mp);
				this.saveAlarm(a);
				this.saveAlarm(newA);
			}
			else{
				a.addMedPacket(mp);
				this.saveAlarm(a);
			}
		}
		else if(a2!=null){
			//already active low alarm alarm
			a2.addMedPacket(mp);
			this.saveAlarm(a2);
		}
		else{
			//no active low hr alarm
			if(mp.getSpo2()<p.getSpo2ThresAlarmLow()){
				//create new low alarm alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.SPO2_ALARM_LOW);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else if(mp.getSpo2()<p.getSpo2ThresWarnLow()){
				//create new low warn alarm
				Alarm newA = new Alarm();
				newA.setPatient(p);
				newA.setTimestamp(d);
				newA.setTriggeredThreshold(Thresholds.SPO2_WARN_LOW);
				newA.addMedPacket(mp);
				this.saveAlarm(newA);
			}
			else{
				//no low SPO2 alarm
			}
		}
	}
}
