package idi.diplom.linda.logic;

import idi.diplom.linda.db.implementation.HibernateUtil;
import idi.diplom.linda.db.interfaces.DAOFactory;
import idi.diplom.linda.db.interfaces.MedPacketDAO;
import idi.diplom.linda.db.interfaces.PatientDAO;
import idi.diplom.linda.exceptions.DatabaseException;
import idi.diplom.linda.exceptions.GeneralException;
import idi.diplom.linda.model.Alarm;
import idi.diplom.linda.model.MedPacket;
import idi.diplom.linda.model.Patient;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.hibernate.HibernateException;

public class MedPacketLogic {
	
	/**
	 * Returns the last medpacket fetched from LifeShirt on the specified patient
	 * @param patient to get monitoring data from
	 * @return the last saved medpacket
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public MedPacket getLast(Patient p) throws DatabaseException, GeneralException{
		MedPacket mp = null;
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			MedPacketDAO mpdao = DAOFactory.instance(DAOFactory.HIBERNATE).getMedPacketDAO();		
			mp = mpdao.findLast(p);		
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();
			
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when calling method getLast()", he);
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
		return mp;
	}
	
	/**
	 * NOT IN USE IN THIS VERSION.
	 * @param a
	 * @return
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public List<MedPacket> getMedPacketsFromIntervalInFront(Alarm a) throws DatabaseException, GeneralException{
		Date toDate = a.getTimestamp();
		Calendar fromTime = Calendar.getInstance();
		fromTime.setTime(toDate);
		fromTime.setTimeInMillis(fromTime.getTimeInMillis()-a.getPatient().getAlarmInterval()*60*1000);
		Date fromDate = fromTime.getTime();
		List<MedPacket> retList = null;
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			MedPacketDAO mpdao = DAOFactory.instance(DAOFactory.HIBERNATE).getMedPacketDAO();		
			retList = mpdao.getMedPacketsBetweenDate(a.getPatient(), fromDate, toDate);	
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when calling method getLast()", he);
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
		return retList;
	}
	
	/**
	 * Returns a list of all medpackets in database on specified patient between the specified dates.
	 * Also only every resolution packets are received.  
	 * @param Patient p
	 * @param fromTime
	 * @param toTime
	 * @param int resolution
	 * @return MedPacket[] medpacketArray
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public MedPacket[] getMedPacketsBetweenDates(Patient p, Date fromTime, Date toTime, int resolution) throws DatabaseException, GeneralException{

		List<MedPacket> retList = new ArrayList<MedPacket>();
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			MedPacketDAO mpdao = DAOFactory.instance(DAOFactory.HIBERNATE).getMedPacketDAO();	
			PatientDAO pdao = DAOFactory.instance(DAOFactory.HIBERNATE).getPatientDAO();
			Patient pap =	pdao.findById(p.getId(), false);
			List<MedPacket> tempList = mpdao.getMedPacketsBetweenDate(pap, fromTime, toTime);	
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();
			
			for(int i=0; i<tempList.size(); i++){
				if (i%resolution == 0)
					retList.add(tempList.get(i));
				
			}
			
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when calling method getMedPacketsBetweenDates()", he);
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
		MedPacket[] mArray = new MedPacket[retList.size()];
		mArray = retList.toArray(mArray);
		return mArray;
	}
	

	
}
