package idi.diplom.linda.logic;

import idi.diplom.linda.db.implementation.HibernateUtil;
import idi.diplom.linda.db.interfaces.DAOFactory;
import idi.diplom.linda.db.interfaces.NoteDAO;
import idi.diplom.linda.db.interfaces.PatientDAO;
import idi.diplom.linda.exceptions.DatabaseException;
import idi.diplom.linda.exceptions.GeneralException;
import idi.diplom.linda.model.Note;
import idi.diplom.linda.model.Patient;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.hibernate.HibernateException;

public class NoteLogic {
	
	/**
	 * Saves the note in database
	 * @param note to be saved
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public void saveNote(Note n) throws DatabaseException, GeneralException{
		TimeZone tz = TimeZone.getDefault();
		Calendar instance = Calendar.getInstance();
		instance.setTimeZone(tz);
		Date saveTime = instance.getTime();
		n.setCreated(saveTime);
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			NoteDAO ndao = DAOFactory.instance(DAOFactory.HIBERNATE).getNoteDAO();		
			PatientDAO pdao = DAOFactory.instance(DAOFactory.HIBERNATE).getPatientDAO();
			Patient pap =	pdao.findById(n.getPatient().getId(), false);
			n.setPatient(pap);
			ndao.makePersistent(n);
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();		
	//		System.out.println("Stored medpacket");
	//		log.info("Stored note");
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when calling method saveNote()", he);			
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
	}
	
	public Note[] getNotes(Patient p, Date fromDate, Date toDate){
		return null;
	}
	
	/**
	 * Receives all notes saved on the specified patient
	 * @param patient to get notes from
	 * @return all saved notes
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public Note[] getAllNotes(Patient p) throws DatabaseException, GeneralException {
		Note[] nArray = null;
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			NoteDAO ndao = DAOFactory.instance(DAOFactory.HIBERNATE).getNoteDAO();
			PatientDAO pdao = DAOFactory.instance(DAOFactory.HIBERNATE).getPatientDAO();
			Patient pap =	pdao.findById(p.getId(), false);
			List<Note> nList = ndao.getAllNotes(pap);			
			nArray = new Note[nList.size()];
			nArray = nList.toArray(nArray);
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();		
		//	log.info("Fetched all patient names");
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when retrieving notes", he);
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
		return nArray;
	
	}
}
