package idi.diplom.linda.logic;

import idi.diplom.linda.db.implementation.HibernateUtil;
import idi.diplom.linda.db.interfaces.DAOFactory;
import idi.diplom.linda.db.interfaces.PatientDAO;
import idi.diplom.linda.exceptions.DatabaseException;
import idi.diplom.linda.exceptions.GeneralException;
import idi.diplom.linda.model.MedPacket;
import idi.diplom.linda.model.Patient;

import java.util.List;

import org.apache.log4j.Logger;
import org.hibernate.HibernateException;

public class PatientLogic {
	
	/**
	 * Saves a patient in the database
	 * @param patient to be saved
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public void savePatient(Patient p) throws DatabaseException, GeneralException{
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			PatientDAO pdao = DAOFactory.instance(DAOFactory.HIBERNATE).getPatientDAO();		
			pdao.makePersistent(p);
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();		
		//	log.info("Stored new patient");
		}
		catch(HibernateException he){
			//log.error("Error caused by hibernate when saving patient", he);
			throw new DatabaseException("Error caused by hibernate when saving patient", he);
		}
		catch(Exception e){
			//log.error("Error when saving patient", e);
			throw new GeneralException("Unknown error", e);
		}
	}
	
	/**
	 * Returns an array of all patients saved in the database
	 * @return patientArray with all saved patients
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public Patient[] getAllPatients() throws DatabaseException, GeneralException{
		Patient[] pArray = null;
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			PatientDAO pdao = DAOFactory.instance(DAOFactory.HIBERNATE).getPatientDAO();		
			List<Patient> pList = pdao.findAll();
			pArray = new Patient[pList.size()];
			pArray = pList.toArray(pArray);
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();		
		//	log.info("Fetched all patient names");
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when saving patient", he);
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
		return pArray;
	}
	
	/**
	 * Retrieves a patient based on the lifeshirtId
	 * @param int lifeshirtId
	 * @return patient with specified lifeshirtId
	 * @throws DatabaseException
	 * @throws GeneralException
	 */
	public Patient getPatient(int lifeshirtId) throws DatabaseException, GeneralException{
		Patient p = null;
		try{
			HibernateUtil.getSessionFactory().getCurrentSession().beginTransaction();
			PatientDAO pdao = DAOFactory.instance(DAOFactory.HIBERNATE).getPatientDAO();	
			p = pdao.findByLifeShirtId(lifeshirtId);
			HibernateUtil.getSessionFactory().getCurrentSession().getTransaction().commit();
		}
		catch(HibernateException he){
			throw new DatabaseException("Error caused by hibernate when saving patient", he);
		}
		catch(Exception e){
			throw new GeneralException("Unknown error", e);
		}
		return p;
	}
}
