using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MobileApp.lifeshirt;
using MobileApp.Logic;
using System.Runtime.InteropServices;

namespace MobileApp
{
    public partial class AlarmForm : Form
    {
        private List<Alarm> alarms;
        private bool isRed;
        private bool isWarning;
        private Form1 mainForm;

        public AlarmForm(Form1 mainForm)
        {
            InitializeComponent();
            alarmLabel.BackColor = Color.DarkGray;
            alarmPanel.BackColor = Color.DarkGray;
            isRed = false;
            isWarning = false;
            this.mainForm = mainForm;
            //playAlarm(@"\..\..\Windows\Ring-WindowsMobile.wma");
            //playAlarm(@"Windows\Standard.wav");
            //IntPtr HINSTANCE = Marshal.GetHINSTANCE(MyObject.GetType().Module);
            playAlarm(MobileApp.Properties.Resources.ding);
            //System.IO.Stream stream = GetType().Assembly.GetManifestResourceStream("ding");
            //label2.Text = stream.Length.ToString();
            startVibrate();
        }

        [DllImport("CoreDll.dll", EntryPoint = "PlaySound", SetLastError = true)]
        //private extern static int PlaySound(string szSound, IntPtr hMod, int flags);
        private extern static int PlaySound(byte[] szSound, IntPtr hMod, int flags);

        private enum SND
        {
        SYNC = 0x0000,
        ASYNC = 0x0001,
        NODEFAULT = 0x0002,
        MEMORY = 0x0004,
        LOOP = 0x0008,
        NOSTOP = 0x0010,
        NOWAIT = 0x00002000,
        ALIAS = 0x00010000,
        ALIAS_ID = 0x00110000,
        FILENAME = 0x00020000,
        RESOURCE = 0x00040004
        }

        public void setAlarms(List<Alarm> alarms)
        {
            this.alarms = alarms;
            showAlarm();
            updateScreen();
        }

        private void playAlarm(byte[] bytes)
        {
            //PlaySound(fileName, IntPtr.Zero, (int)(SND.ASYNC | SND.LOOP));
            try
            {
                PlaySound(bytes, IntPtr.Zero, (int)(SND.ASYNC | SND.LOOP | SND.MEMORY));
            }
            //PlaySound(fileName, MobileApp.Properties.Resources.ding, (int)(SND.ASYNC));
            //PlaySound(fileName, MobileApp.Properties.Resources.ResourceManager, (int)(SND.ASYNC | SND.LOOP | SND.RESOURCE));
            catch (Exception e)
            {
                //unable to play sound
            }
        }

        [DllImport("aygshell.dll", EntryPoint = "Vibrate", SetLastError = true)]
        private static extern int Vibrate(int cvn, IntPtr rgvn, bool fRepeat, uint dwTimeout);
        public bool startVibrate()
        {
            try
            {
                int result = Vibrate(0, IntPtr.Zero, true, 0xffffffff);
                //int result = Vibrate(0, IntPtr.Zero, true, Double.PositiveInfinity);
                if (result != 0)
                {
                    return false;
                }
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }

        [DllImport("aygshell.dll", EntryPoint = "VibrateStop", SetLastError = true)]
        private static extern int VibrateStop();
        public static bool stopVibrate()
        {
            try
            {
                int result = VibrateStop();
                if (result != 0)
                {
                    return false;
                }
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }

        private void showAlarm()
        {
            //Vibrate(0, NULL, TRUE, INFINITE);
            String text = "";
            bool testIfWarning = true;
            foreach (Alarm a in alarms)
            {
                if (!AlarmType.isWarning(a.triggeredThreshold))
                {
                    testIfWarning = false;
                }
                String alarmText = AlarmType.GetString(a.triggeredThreshold);
                DateTime date = a.timestamp.Value.ToLocalTime();
                //DateTime date2 = date.AddHours(1);
                text += "\n" + alarmText;
                text += "\n(Created: " + date.ToString() + ")\n";
            }
            if (testIfWarning)
            {
                isWarning = true;
            }
            else
            {
                isWarning = false;
            }
            alarmLabel.Text = text;
        }

        private void updateScreen()
        {
            if (isWarning)
            {
                alarmLabel.BackColor = Color.Yellow;
                alarmPanel.BackColor = Color.Yellow;
            }
            else
            {
                alarmLabel.BackColor = Color.Red;
                alarmPanel.BackColor = Color.Red;
            }
            if (isRed)
            {  
                label1.ForeColor = Color.Black;   
                alarmLabel.ForeColor = Color.Black;
                isRed = false;
            }
            else
            {
                label1.ForeColor = Color.White;
                alarmLabel.ForeColor = Color.White;
                isRed = true;
            }
        }

        public List<Alarm> Alarms
        {
            get
            {
                return alarms;
            }
        }
        public DialogResult Dialog
        {
            get
            {
                return this.DialogResult;
            }
        }
        private void dismissItem_Click(object sender, EventArgs e)
        {
            playAlarm(null);
            stopVibrate();
            mainForm.enableTimer(false);
            dismissPanel.Visible = true;
            dismissLabel.Visible = true;
            dismissTextBox.Visible = true;
            dismissTextBox.Focus();
            OkItem.Enabled = true;
            dismissItem.Enabled = false;
        }

        public void stopAlarms()
        {
            playAlarm(null);
            stopVibrate();
        }

        public void setFinished()
        {
            playAlarm(null);
            stopVibrate();
            this.DialogResult = System.Windows.Forms.DialogResult.OK;
        }

        private void OkItem_Click(object sender, EventArgs e)
        {
            foreach (Alarm a in alarms)
            {
                a.dismissNote = dismissTextBox.Text;
                //a.dismissTime = DateTime.Now.ToUniversalTime();
            }
            mainForm.enableTimer(true);
            this.DialogResult = System.Windows.Forms.DialogResult.OK;
        }
    }
}