using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Media;
using System.Text;
using System.Timers;
using System.Windows.Forms;
using PcClient.BusinessLogic;
using PcClient.lifeshirt;
using Timer=System.Windows.Forms.Timer;
using System.Threading;

namespace PcClient
{
    public partial class AlarmForm : Form
    {
        private bool isRed;
        private bool isWarning;
        private Timer timer;
        private List<Alarm> alarms;

        private SoundPlayer alarmSound;
        private SoundPlayer warningSound;
        private bool soundPlaying;

        public AlarmForm(List<Alarm> alarms, SoundPlayer alarmSound, SoundPlayer warningSound)
        {
            this.alarmSound = alarmSound;
            this.warningSound = warningSound;
            soundPlaying = false;
         

            Paint += new System.Windows.Forms.PaintEventHandler(AlarmForm_Paint);
            InitializeComponent();
            this.alarms = alarms;
            UpdateAlarms();

            Rectangle screenSize = System.Windows.Forms.Screen.PrimaryScreen.Bounds;
            Size = new Size(screenSize.Width, screenSize.Height);

            createPanel();

            startAnimation();
         
        }

        public List<Alarm> Alarms
        {
            get { return alarms; }
            set { alarms = value; }
        }

        private delegate void UpdateAlarmsDelegate();
        public void UpdateAlarms()
        {
            if (InvokeRequired){
                Invoke(new UpdateAlarmsDelegate(UpdateAlarms), new object[] {  });
                return;
            }

            //alarms dismissed elsewhere
            if (alarms.Count == 0)
            {
                stopAnimation();
                dismissedPanel.Visible = true;
                dismissedOkButton.Focus();
                return;
            }

            isWarning = true;
            foreach (Alarm alarm in alarms)
            {
                if (!AlarmType.isWarning(alarm.triggeredThreshold))
                    isWarning = false;
            }

            //TODO lag slik at den animerer forskjellig p warning og p alarm
            if (isWarning)
            {
                //animer warning
                headerLabel.Text = "WARNING";
                if (!soundPlaying){
                    warningSound.PlayLooping();
                    soundPlaying = true;
                }
                
            }
            else //isAlarm (ihvertfall n)
            {
                headerLabel.Text = "ALARM";
                if (!soundPlaying){
                    alarmSound.PlayLooping();
                    soundPlaying = true;
                }
                //lag alarm
                //vis alle alarmtekster                
            }
            listBox1.Items.Clear();
            foreach (Alarm alarm in alarms)
            {
                try
                {
                    String time = "";
                    if (alarm.timestamp.HasValue)
                        time = alarm.timestamp.Value.ToLocalTime().ToShortTimeString();
                    String text = string.Format("{0} {1}", time, AlarmType.GetString(alarm.triggeredThreshold));                    
                    listBox1.Items.Add(text);                    
                    //Console.WriteLine(text);
                }
                catch (Exception e)
                {
                    //TODO CrossThreadException !! :/
                }
            }
            
        }


        private void createPanel()
        {
            panel1.BackColor = Color.DarkGray;
            int x = (Size.Width/2) - (panel1.Width/2);
            int y = (Size.Height/2) - (panel1.Height/2);
            panel1.Location = new Point(x, y);
            /*   Label label = new Label();
               label.Text = "testestests";
              panel1.Controls.Add(label);
               */
            //Panel p = new Panel();

            //p.BackColor = Color.Purple;
            //this.components.Add(p);
            //   panel1.Size = new Size(400, 200);

            // panel1.Location.X = 500;

            //panel1.Location.X 
        }

        private void startAnimation()
        {
            BackColor = Color.Gray;
            isRed = false;
            timer = new Timer();
            timer.Interval = 350;
            timer.Tick += animation_Tick;
            timer.Start();
        }

        private void animation_Tick(object sender, EventArgs e)
        {
            if (isWarning){
                if (isRed)
                    BackColor = Color.Yellow;
                else 
                    BackColor = Color.YellowGreen;
            }
            else
            {
                if (isRed)
                    BackColor = Color.Red;
                else
                    BackColor = Color.DarkRed;
            }
            isRed = !isRed;
        }

        private void AlarmForm_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            //     Graphics g = this.CreateGraphics();

            //  g.DrawRectangle(new Pen(Color.FromArgb(80, 0, 0, 0), 50), 50, 50, 250, 250 );
        }

        private void stopAnimation()
        {

            timer.Stop();
            warningSound.Stop();
            alarmSound.Stop();
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            //  this.Close();
            stopAnimation();

            DismissDialog dd = new DismissDialog();
            if (dd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                foreach (Alarm alarm in alarms)
                {
                    alarm.dismissNote = dd.GetDismissNote();
                    alarm.dismissTime = DateTime.Now.ToUniversalTime();

                }
                DialogResult = System.Windows.Forms.DialogResult.OK;
               // Console.WriteLine("AlarmForm: " + Thread.CurrentThread.Name );
                MainForm mform = (MainForm) Owner;
                
                mform.AcceptingAlarms = false;
                mform.storeAlarmsBGworker.RunWorkerAsync(alarms);
                Close();
                // Application.Exit();     
            }
        }

        private void dismissedOkButton_Click(object sender, EventArgs e)
        {
            MainForm mform = (MainForm)Owner;

            mform.AcceptingAlarms = false;
            mform.storeAlarmsBGworker.RunWorkerAsync(alarms);
            Close();
        }
    }
}