using System;
using System.Collections.Generic;
using System.Text;
using System.Timers;
using PcClient;
//using DomainModel;
using PcClient.lifeshirt;

namespace PcClient.BusinessLogic
{
    internal class WorkerThread
    {
        private MainForm mForm;
        private WSLogic wsLogic;
        private Patient patient;
        private List<Alarm> storedAlarms;
        private MedPacket lastMedPacket;
        private Timer timer;

        private int nofMissedPackets;

        public WorkerThread(MainForm mForm, Patient patient)
        {
            this.mForm = mForm;
            this.patient = patient;
            nofMissedPackets = 0;
            wsLogic = new WSLogic();
            Initialize();
        }

        public void stopWorking()
        {
            timer.Stop();   
        }

        public void Initialize()
        {
            timer = new Timer();
            timer.Elapsed += new ElapsedEventHandler(doWork);
         
            timer.Interval = 1000;
            timer.Start();
        }

        private void doWork(object sender, ElapsedEventArgs e)
        {
            MedPacket mp = wsLogic.GetLastMedPacket(patient);

            if (nofMissedPackets > 30)
            {
                nofMissedPackets = 0;
               // Console.WriteLine("sending no connection alarm");
             
                Alarm a = new Alarm();
                a.triggeredThreshold = AlarmType.NOT_RECEIVING_MEDPACKETS;
                a.patient = patient;
                a.timestamp = DateTime.Now;
                a.medPackets = null;
                List<Alarm> alist = new List<Alarm>();
                alist.Add(a);
                storedAlarms = alist;
                mForm.HandleAlarms(alist);
                return;
            }
               

            if (mp == null)
            {              
                nofMissedPackets++;               
                return;
            }
            if (lastMedPacket==null || ( mp.id != lastMedPacket.id)) //check that we received a new medpacket
            {
                nofMissedPackets = 0;
                List<Alarm> alarms = wsLogic.GetActiveAlarms(patient);
               
                //alarms dismissed elsewhere
                //if (storedAlarms!=null && storedAlarms.Count > 0 && alarms.Count==0 && storedAlarms[0].dismissTime != null)
                //{
                //    storedAlarms = alarms;
                //    mForm.HandleAlarms(alarms);
                //}
                //checks to see if the list contains new alarms, will update alarmform so to reflect new alarms
                if (alarms != null && containsNewAlarms(alarms))
                {                  
                    foreach(Alarm a in alarms)
                         Console.Write(a.id + " " + AlarmType.GetString(a.triggeredThreshold));
                    Console.WriteLine();
                    
                    
                    storedAlarms = alarms;
                   // if (alarms.Count > 0)
                        mForm.HandleAlarms(alarms);                                       
                }
                
                lastMedPacket = mp;
                mForm.HandleMedPacket(mp);
            }
            else
            {//not receiving new medpackets
                nofMissedPackets++;
                Console.WriteLine("No medpacket received");
            }
        }

        private bool containsNewAlarms(List<Alarm> newList)
        {
            if (storedAlarms == null)
                return true;

            if (newList.Count != storedAlarms.Count)
                return true;

            foreach (Alarm alarm in newList)
            {
                bool contains = false;
                foreach (Alarm storedAlarm in storedAlarms)
                {
                    if (alarm.id == storedAlarm.id)
                        contains = true;
                }

                if (!contains) //then the new list has an alarm that's not registered yet
                    return true;
            }

            return false;
        }
    }
}