using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PcClient.BusinessLogic;
using PcClient.lifeshirt;
using System.Threading;
using ZedGraph;

namespace PcClient
{
    public partial class GraphForm : Form
    {
        private Patient patient;
        private WSLogic wslogic;
        private DateTime startDate;
        private List<MedPacket> medpackets;
        

        public GraphForm(Patient patient)
        {
            this.patient = patient;
            wslogic = new WSLogic();
         
            // backgroundWorker1.ProgressChanged
            InitializeComponent();

            fromBox.Text = DateTime.Now.AddHours(-2).ToString("dd.MM.yy HH:mm");
            toBox.Text = DateTime.Now.ToString("dd.MM.yy HH:mm");

           // createGraphs();
            
        }

        private void generateButton_Click(object sender, EventArgs e)
        {
            generateButton.Enabled = false;
            cancelButton.Enabled = true;
            startDate = DateTime.Now;
            toolStripStatusLabel1.Text = "Retrieving from DB ...";
            toolStripProgressBar1.Visible = true;
            statusStrip1.Visible = true;
            timer1.Start();
            
            try
            {
                DateTime fromDate = DateTime.Parse(fromBox.Text).ToUniversalTime();
                DateTime toDate = DateTime.Parse(toBox.Text).ToUniversalTime();
                generateBG.RunWorkerAsync(new object[] { patient, fromDate, toDate, getResolution() });
            }
            catch (FormatException ex)
            {
                MessageBox.Show("Please ensure proper formatting of dates");
            }
           
            //List<MedPacket> mps = wslogic.GetMedPackets(patient, DateTime.Now.AddDays(-1), DateTime.Now);
            //Console.WriteLine(mps.Count);
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            if (generateBG.IsBusy)
                generateBG.CancelAsync();
        }

        private void generateBG_DoWork(object sender, DoWorkEventArgs e)
        {
            //PointPairList list1 = new PointPairList();
            object[] args = (object[]) e.Argument;
            Patient p = (Patient)args[0];
            DateTime from = (DateTime)args[1];
            DateTime to = (DateTime)args[2];
            int resolution = (int) args[3];
            //inni lkka
            //for (int i = 0; i < 10; i++)
            //{
            //    if (generateBG.CancellationPending)
            //    {
            //        e.Cancel = true;
            //        return;
            //    }
            //    generateBG.ReportProgress(i*10);
            //    //generateBG.ReportProgress(percentprogress, userstate);
            //    Thread.Sleep(1000);
            //}
            //TODO fiks ordentlig her
            
            List<MedPacket> mplist = wslogic.GetMedPackets(patient, from, to, resolution);
            foreach (MedPacket mp in mplist)
            {
                mp.timestamp = mp.timestamp.Value.ToLocalTime();
            }
            e.Result = mplist;
            //legg ferdig utregnet greie i e.Result
            
             
        }

        private void generateBG_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {

            //kast om e.UserState, hent ut relevant info, oppdater GUI.
            Console.WriteLine("Reached " + e.ProgressPercentage + "%");
        }

        private void generateBG_Completed(object sender, RunWorkerCompletedEventArgs e)
        {
            if (e.Cancelled)
                Console.WriteLine("Was cancelled");
            else if (e.Error != null)
                MessageBox.Show("Error occured: " + e.Error.Message);
            else{
                Console.WriteLine("Complete!");
                // hent svar fra e.result, m kastes om.
                medpackets = (List<MedPacket>) e.Result;
                Console.WriteLine("medpackets count: " + medpackets.Count);
                if (medpackets.Count == 0)
                    MessageBox.Show("No MedPackets found in given time interval");
                else
                    createGraphs();


            }



            generateButton.Enabled = true;
            cancelButton.Enabled = false;
            statusStrip1.Visible = false;
        }

        private void createGraphs()
        {
            z1.Visible = true;

            MasterPane master = z1.MasterPane;

            // Fill the background
            master.Fill = new Fill(Color.White, Color.FromArgb(220, 220, 255), 45.0f);
            // Clear out the initial GraphPane
            master.PaneList.Clear();

            // Show the masterpane title
            //master.Title.IsVisible = true;
            //master.Title.Text = "Synchronized Panes Demo";

            // Leave no margin around the masterpane, and only a small gap between panes
            master.Margin.All = 0;
            master.InnerPaneGap = 5;

            // The titles for the individual GraphPanes
            string[] yLabels = { "Heart rate", "Breath rate", "SpO2", "Temperature", "Motion", "HeatIdx", "Posture"  };

            //string[] yLabels = { "Heart rate", "Breath rate", "SpO2" };
           // List<string> yLabels = new List<string>();

            bool[] bGraphList = {false, false, false, false, false, false, false};
            if (hrCheckbox.Checked) bGraphList[0] = true;
            if (brCheckbox.Checked) bGraphList[1] = true;
            if (spo2Checkbox.Checked) bGraphList[2] = true;
            if (tempCheckbox.Checked) bGraphList[3] = true;
            if (motionCheckbox.Checked) bGraphList[4] = true;
            if (heatCheckbox.Checked) bGraphList[5] = true;
            if (postureCheckbox.Checked) bGraphList[6] = true;

            int nofGraphs = 0;
            int nofGraphsCreated = 0;
            foreach (bool b in bGraphList)
            {
                if (b)
                    nofGraphs++;
            }


            ColorSymbolRotator rotator = new ColorSymbolRotator();

            for (int j = 0; j < yLabels.Length; j++)
            {
                if (!bGraphList[j])
                    continue;

                nofGraphsCreated++;

                // Create a new graph -- dimensions to be set later by MasterPane Layout
                GraphPane myPaneT = new GraphPane(new Rectangle(10, 10, 10, 10),
                   "",
                   "Time",
                   yLabels[j]);

                myPaneT.Fill.IsVisible = false;

                // Fill the Chart background
                myPaneT.Chart.Fill = new Fill(Color.White, Color.LightYellow, 45.0F);
                // Set the BaseDimension, so fonts are scaled a little bigger
                myPaneT.BaseDimension = 3.0F;

                // Hide the XAxis scale and title               
                myPaneT.XAxis.Title.IsVisible = false;
                myPaneT.XAxis.Scale.IsVisible = false;

                // Hide the legend, border, and GraphPane title
                myPaneT.Legend.IsVisible = false;
                myPaneT.Border.IsVisible = false;
                myPaneT.Title.IsVisible = false;
                // Get rid of the tics that are outside the chart rect
                myPaneT.XAxis.MajorTic.IsOutside = false;
                myPaneT.XAxis.MinorTic.IsOutside = false;
                // Show the X grids
                myPaneT.XAxis.MajorGrid.IsVisible = true;
                myPaneT.XAxis.MinorGrid.IsVisible = true;
                // Remove all margins
                myPaneT.Margin.All = 0;
                // Except, leave some top margin on the first GraphPane
                if (j == 0)
                    myPaneT.Margin.Top = 10;
                else
                    myPaneT.Margin.Top = 13;
                // And some bottom margin on the last GraphPane
                // Also, show the X title and scale on the last GraphPane only
                if (nofGraphsCreated == nofGraphs)
                {
                    myPaneT.XAxis.Title.IsVisible = true;
                    myPaneT.XAxis.Scale.IsVisible = true;

                    myPaneT.Margin.Bottom = 10;
                }
                

                //if (j > 0)
                //    myPaneT.YAxis.Scale.IsSkipLastLabel = true;

                // This sets the minimum amount of space for the left and right side, respectively
                // The reason for this is so that the ChartRect's all end up being the same size.
                myPaneT.YAxis.MinSpace = 80;
                myPaneT.Y2Axis.MinSpace = 20;

                // Make up some data arrays based on the Sine function
/*
                double x, y;
                PointPairList list = new PointPairList();
                for (int i = 0; i < 3600; i++)
                {
                    x = (double)i + 5 + j * 3;
                    y = (j + 1) * (j + 1) * 10 * (1.5 + Math.Sin((double)i * 0.2 + (double)j));
                    list.Add(x, y);
                }
               
                LineItem myCurve = myPaneT.AddCurve("Type " + j.ToString(),
                   list, rotator.NextColor, rotator.NextSymbol);
*/


                DataSourcePointList dspl = new DataSourcePointList();
                
                // Specify the table as the data source
                dspl.DataSource = medpackets;
                // The X data will come from the "OrderDate" column
                dspl.XDataMember = "timestamp";
                myPaneT.XAxis.Type = AxisType.Date;
                
               
                //myPaneT.XAxis.Scale.MinorStep = 30;
                //myPaneT.XAxis.Scale.MinorUnit = DateUnit.Minute;
                //myPaneT.XAxis.Scale.MajorStep = 1;
                //myPaneT.XAxis.Scale.MajorUnit = DateUnit.Day;
                //myPaneT.XAxis.Scale.MaxGrace = 0;
                //myPaneT.XAxis.Scale.MinGrace = 0;
               
                
                switch (j)
                {
                    case 0:
                        dspl.YDataMember = "hr";
                        myPaneT.YAxis.Scale.Min = 0;
                        myPaneT.YAxis.Scale.Max = 200;
                        // Create a band of green to show the highlighted region
                        BoxObj box = new BoxObj(0.0, patient.hrThresWarnHigh, 1.0, patient.hrThresWarnHigh-patient.hrThresWarnLow, Color.Empty,
                            Color.FromArgb(150, Color.LightGreen));
                        
                        // Use CoordType.XChartFractionYScale, so that Y values are regular scale values, and
                        // X values are chart fraction, ranging from 0 to 1
                        box.Location.CoordinateFrame = CoordType.XChartFractionYScale;
                        box.Fill = new Fill(Color.White, Color.FromArgb(200, Color.LightGreen), 45.0F);
                        box.ZOrder = ZOrder.F_BehindGrid;
                        box.IsClippedToChartRect = true;
                        myPaneT.GraphObjList.Add(box);
                        
                        break;
                    case 1:
                        dspl.YDataMember = "br";
                        //myPaneT.YAxis.Scale.Min = 0;
                        //myPaneT.YAxis.Scale.Max = 40;

                        BoxObj box2 = new BoxObj(0.0, patient.brThresWarnHigh, 1.0, patient.brThresWarnHigh - patient.brThresWarnLow, Color.Empty,
    Color.FromArgb(150, Color.LightGreen));

                        // Use CoordType.XChartFractionYScale, so that Y values are regular scale values, and
                        // X values are chart fraction, ranging from 0 to 1
                        box2.Location.CoordinateFrame = CoordType.XChartFractionYScale;
                        box2.Fill = new Fill(Color.White, Color.FromArgb(200, Color.LightGreen), 45.0F);
                        box2.ZOrder = ZOrder.F_BehindGrid;
                        box2.IsClippedToChartRect = true;
                        myPaneT.GraphObjList.Add(box2);

                        break;
                    case 2:
                        dspl.YDataMember = "spo2";
                        myPaneT.YAxis.Scale.Min = 80;
                        myPaneT.YAxis.Scale.Max = 100;
                        BoxObj box3 = new BoxObj(0.0, 100, 1.0, 100 - patient.spo2ThresWarnLow, Color.Empty,
    Color.FromArgb(150, Color.LightGreen));

                        // Use CoordType.XChartFractionYScale, so that Y values are regular scale values, and
                        // X values are chart fraction, ranging from 0 to 1
                        box3.Location.CoordinateFrame = CoordType.XChartFractionYScale;
                        box3.Fill = new Fill(Color.White, Color.FromArgb(200, Color.LightGreen), 45.0F);
                        box3.ZOrder = ZOrder.F_BehindGrid;
                        box3.IsClippedToChartRect = true;
                        myPaneT.GraphObjList.Add(box3);

                        break;                    
                    case 3:
                        dspl.YDataMember = "temp";
                        break;
                    case 4:
                        dspl.YDataMember = "motion";
                        break;
                    case 5:
                        dspl.YDataMember = "heatIdx";
                        break;
                    case 6:
                        dspl.YDataMember = "posture";
                        break;
                }
                
                
                // The Z data are not used
                dspl.ZDataMember = null;
                

                
                LineItem myCurve = myPaneT.AddCurve(yLabels[j], dspl, rotator.NextColor, SymbolType.None);
                
                myCurve.Line.IsOptimizedDraw = true;
               // myCurve.Symbol.Fill = new Fill(Color.White);
                
                
                //myPaneT.XAxis.Scale.Min = 0;
               myPaneT.XAxis.Scale.Min = XDate.DateTimeToXLDate(medpackets[0].timestamp.Value);
                if (medpackets.Count > 150)
                    myPaneT.XAxis.Scale.Max = XDate.DateTimeToXLDate(medpackets[150].timestamp.Value);
                else
                    myPaneT.XAxis.Scale.Max = XDate.DateTimeToXLDate(medpackets[medpackets.Count-1].timestamp.Value);
                //myPaneT.XAxis.Scale.Max = XDate.DateTimeToXLDate(medpackets[2000].timestamp.Value);
                //myPaneT.XAxis.Scale.FormatAuto = true;
                myPaneT.XAxis.Scale.Format = "HH:mm";
                

                master.Add(myPaneT);
            }

            using (Graphics g = this.CreateGraphics())
            {
            
                master.SetLayout(g, PaneLayout.SingleColumn);
                master.AxisChange(g);
                
               
                // Synchronize the Axes
                z1.ScrollMinX = XDate.DateTimeToXLDate(medpackets[0].timestamp.Value);
                z1.ScrollMaxX = XDate.DateTimeToXLDate(medpackets[medpackets.Count-1].timestamp.Value);
                z1.ScrollGrace = 0.01;
               // z1.GraphPane.XAxis.Scale.Min = XDate.DateTimeToXLDate(medpackets[0].timestamp.Value);
               // z1.GraphPane.XAxis.Scale.Max = XDate.DateTimeToXLDate(medpackets[100].timestamp.Value);
                

                z1.IsShowHScrollBar = true;
                z1.IsSynchronizeXAxes = true;
                z1.IsShowPointValues = true;
                
                //z1.IsAutoScrollRange = true;
                

                
                // Horizontal pan and zoom allowed
                z1.IsEnableHPan = true;
                z1.IsEnableHZoom = true;

                // Vertical pan and zoom not allowed
                z1.IsEnableVPan = false;
                z1.IsEnableVZoom = false;



                //g.Dispose();
            }
            z1.Invalidate();

          
        }

        private int getResolution()
        {
            if (radioOneSecond.Checked) return 1;
            else if (radioFiveSeconds.Checked) return 5;
            else if (radioFiftSeconds.Checked) return 15;
            else if (radioThirtySeconds.Checked) return 30;
            else if (radioOneMinutes.Checked) return 60;
            else if (radioFiveMinutes.Checked) return 60 * 5;
            else
                return 1;
        }



        //private void z1Scroll_Done(ZedGraphControl sender, ScrollBar scrollBar, ZoomState oldState, ZoomState newState)
        //{
        //    z1.AxisChange();
        //    z1.Invalidate();
        //}


    }
}