using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Media;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using PcClient.BusinessLogic;
//using DomainModel;
using PcClient.lifeshirt;

namespace PcClient
{
    public partial class MainForm : Form
    {
        private WorkerThread worker;
        private List<Patient> patientList;
        private AlarmForm alarmForm;
        private WSLogic wsLogic;
        private Patient activePatient;
        private bool acceptingAlarms;
        private bool isMonitoring;
        private bool isReceivingPackets;

        private SoundPlayer alarmSound;
        private SoundPlayer warningSound;


        public MainForm()
        {
            acceptingAlarms = true;
            isReceivingPackets = true;
            isMonitoring = false;
            alarmSound = new SoundPlayer("ding.wav");
            alarmSound.LoadAsync();
            warningSound = new SoundPlayer("ding.wav");
            warningSound.LoadAsync();

            Thread.CurrentThread.Name = "main";
            InitializeComponent();
            wsLogic = new WSLogic();
            //    patientList = wsLogic.GetAllPatients();

            toolStripProgressLabel.Text = "Retrieving patients list";
            toolStripProgressLabel.Visible = true;
            toolStripProgressBar1.Visible = true;
            getPatientsBGworker.RunWorkerAsync();
        }

        public bool AcceptingAlarms
        {
            get { return acceptingAlarms; }
            set { acceptingAlarms = value; }
        }

        private void SetThresholdLabels()
        {
            hrAlarmHigh.Text = string.Format("Threshold alarm high: {0}", activePatient.hrThresAlarmHigh);
            hrWarnHigh.Text = string.Format("Threshold warn high: {0}", activePatient.hrThresWarnHigh);
            hrWarnLow.Text = string.Format("Threshold warn low: {0}", activePatient.hrThresWarnLow);
            hrAlarmLow.Text = string.Format("Threshold alarm low: {0}", activePatient.hrThresAlarmLow);

            brAlarmHigh.Text = string.Format("Threshold alarm high: {0}", activePatient.brThresAlarmHigh);
            brWarnHigh.Text = string.Format("Threshold warn high: {0}", activePatient.brThresWarnHigh);
            brWarnLow.Text = string.Format("Threshold warn low: {0}", activePatient.brThresWarnLow);
            brAlarmLow.Text = string.Format("Threshold alarm low: {0}", activePatient.brThresAlarmLow);

            spo2WarnLow.Text = string.Format("Threshold warn low: {0}", activePatient.spo2ThresWarnLow);
            spo2AlarmLow.Text = string.Format("Threshold alarm low: {0}", activePatient.spo2ThresAlarmLow);
        }
        private void StartPatientMonitoring()
        {
            panel1.Visible = true;
            noConnectionPanel.Visible = true;
            isMonitoring = true;

            SetThresholdLabels();
            

            monitoringStatusLabel.Text = "Monitoring ON";
            monitoringStatusLabel.ForeColor = Color.Green;

            
            monitoringStatusButton.Text = "Stop monitoring";

            if (worker != null)
                worker.stopWorking();
            
            worker = new WorkerThread(this, activePatient);
        }

        private void StopPatientMonitoring()
        {
            isMonitoring = false;
            if (worker != null)
                worker.stopWorking();

            monitoringStatusLabel.ForeColor = Color.Red;
            monitoringStatusLabel.Text = "Monitoring OFF";

            monitoringStatusButton.Text = "Start monitoring";

            isReceivingPackets = true;
            noConnectionPanel.Visible = false;
            panel1.Visible = false;
            
        }


        #region Handle methods

        private delegate void HandleMedPacketDelegate(MedPacket mp);
        public void HandleMedPacket(MedPacket mp)
        {
            if (mp == null)
                return;

          
            if (InvokeRequired)
            {
                Invoke(new HandleMedPacketDelegate(HandleMedPacket), new object[] {mp});
                return;
            }

            noConnectionPanel.Visible = false;
            isReceivingPackets = true;


            //Console.WriteLine("received medpacket: " + mp.id);
            mpIdLabel.Text = "MedPacket id: " + mp.id;
            hrLbl.Text = "HR: " + mp.hr;
            brLbl.Text = "BR: " + mp.br;
            spo2Lbl.Text = "SPO2: " + mp.spo2 + "%";
            postureLbl.Text = "Posture: " + mp.posture;
            motionLbl.Text = "Motion: " + mp.motion;
            tempLbl.Text = "Temp: " + mp.temp.ToString("0.0") + "C";
            timestampLabel.Text = "Timestamp: " + mp.timestamp.Value.ToLocalTime();
            
            
        }

        private delegate void HandleAlarmsDelegate(List<Alarm> alarms);
        public void HandleAlarms(List<Alarm> alarms)
        {
            if (!acceptingAlarms || alarms==null) 
                return;

          
            if (InvokeRequired)
            {
                Invoke(new HandleAlarmsDelegate(HandleAlarms), new object[] {alarms});
                return;
            }
            //together with dismiss alarm
            if (alarmForm == null && alarms.Count == 0)
                return;

            if (!isReceivingPackets)
                return;

            if (alarms.Count == 1 && alarms[0].triggeredThreshold == AlarmType.NOT_RECEIVING_MEDPACKETS)
            {
                isReceivingPackets = false;
                noConnectionPanel.Visible = true;
            }

            //kontakt alarm form, oppdater alarm
            if (alarmForm == null)
            {

                alarmForm = new AlarmForm(alarms, alarmSound, warningSound);
                alarmForm.Owner = this;
                

                toolStripProgressLabel.Text = "Storing alarms";
                toolStripProgressLabel.Visible = true;
                toolStripProgressBar1.Visible = true;

                alarmForm.Show(this);      
                alarmForm.Activate();
            }
            else
            {
                //eksisterer allerede et alert vindu, sender nye alarmer slik at vinduet kan oppdateres


                //if (alarms.Count == 0) //alarms dismissed
                //{
                //    toolStripProgressLabel.Visible = false;
                //    toolStripProgressBar1.Visible = false;
                //}


                Console.Write("MainForm: received new alarms list : ");
                foreach (Alarm alarm in alarms)
                {
                    Console.Write(alarm.id + " " + AlarmType.GetString(alarm.triggeredThreshold) + ", ");
                }
                Console.WriteLine();              
                alarmForm.Alarms = alarms;
                alarmForm.UpdateAlarms();
            }
        }

        #endregion

        #region Background workers methods

        private void storeAlarmsBGworker_DoWork(object sender, DoWorkEventArgs e)
        {
            List<Alarm> alarmsToBeStored = (List<Alarm>) e.Argument;
            foreach (Alarm alarm in alarmsToBeStored)
            {                
                wsLogic.dismissAlarms(alarmsToBeStored);
            }            
        }

        private void storeAlarmsBGworker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            acceptingAlarms = true;
            alarmForm = null;
            toolStripProgressLabel.Visible = false;
            toolStripProgressBar1.Visible = false;
            if (e.Error != null)
                MessageBox.Show("Error occured while trying to save dismissed alarms: " + e.Error.Message);
        }

        private void getPatientsBGworker_DoWork(object sender, DoWorkEventArgs e)
        {
            patientList = wsLogic.GetAllPatients();
        }

        private void getPatientsBGworker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            toolStripProgressLabel.Visible = false;
            toolStripProgressBar1.Visible = false;

            if (e.Error != null)
                MessageBox.Show("Could not retrieve patient list from server: " + e.Error.Message);
            else selectPatientToolStripMenuItem.Enabled = true;
            monitoringStatusButton.Enabled = true;
        }

        #endregion
      
        #region Toolstrip click methods


        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // MessageBox.Show("Diplomoppgave 2008", "About", MessageBoxButtons.OK, MessageBoxIcon.Information);
            AboutBox1 abt = new AboutBox1();
            abt.ShowDialog();
        }

        private void selectPatientToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SelectPatientDialog dlg = new SelectPatientDialog(patientList);
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                activePatient = dlg.Patient;
                patientNameLabel.Text = "Patient: " + activePatient.name;
                patientNameLabel.Visible = true;

                createNoteToolStripMenuItem.Enabled = true;
                showGraphWindowToolStripMenuItem.Enabled = true;
                editPatientToolStripMenuItem.Enabled = true;
                viewAllNotesToolStripMenuItem.Enabled = true;
                if (dlg.MonitorAlarms)
                {                  
                   StartPatientMonitoring();
                }
                else
                {
                    if (isMonitoring)
                        StopPatientMonitoring();

                    monitoringStatusLabel.Text = "Monitoring OFF";
                    monitoringStatusLabel.ForeColor = Color.Red;

                    monitoringStatusButton.Text = "Start monitoring";
                }
            }
        }

        private void showGraphWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GraphForm gf = new GraphForm(activePatient);
            gf.Show();
        }

        private void createNoteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NoteDialog nd = new NoteDialog(activePatient);
            if (nd.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
               
            }
        }

        private void newPatientToolStripMenuItem_Click(object sender, EventArgs e)
        {
            EditPatientDialog epd = new EditPatientDialog(null);
            if (epd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                  
                    MessageBox.Show("Patient saved to server, please select the patient under \nPatient/Select patient");
                    getPatientsBGworker.RunWorkerAsync();                   
                }
                catch (Exception ex)
                {
                    MessageBox.Show("An error occurred, try again. (" + ex.Message + ")");
                }
            }
        }

        private void editPatientToolStripMenuItem_Click(object sender, EventArgs e)
        {
            EditPatientDialog epd = new EditPatientDialog(activePatient);
            if (epd.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
            {
                SetThresholdLabels();
            }
        }

        #endregion

        private void monitoringStatusButton_Click(object sender, EventArgs e)
        {
            if (activePatient == null)
                selectPatientToolStripMenuItem_Click(sender, e);
            else
            {
                if (isMonitoring)
                {
                    StopPatientMonitoring();
                }
                else
                {
                    StartPatientMonitoring();
                }

            }

        }

        private void viewAllNotesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ViewNotesForm vnf = new ViewNotesForm(activePatient);
            vnf.Show();
        }



     

    }
}